@@ -1,4 +1,4 @@
-"""Define project metadata
+"""Project metadata.
 """
 
 __all__ = [
@@ -12,12 +12,10 @@ __all__ = [
 ]
 
 __title__ = 'alchy'
-__summary__ = 'The declarative companion to SQLAlchemy'
+__summary__ = ('A SQLAlchemy extension for its declarative ORM that provides '
+               'enhancements for model classes, queries, and sessions.')
 __url__ = 'https://github.com/dgilland/alchy'
-
-__version__ = '0.13.3'
-
+__version__ = '1.0.0'
 __author__ = 'Derrick Gilland'
 __email__ = 'dgilland@gmail.com'
-
 __license__ = 'MIT License'
