@@ -26,33 +26,41 @@ except ImportError:  # pragma: no cover
             self()
 
 from ._compat import iteritems
-from .utils import join_subquery_on_columns
+
+
+__all__ = [
+    'Query',
+    'QueryModel',
+    'QueryProperty',
+    'Pagination',
+]
 
 
 class Query(orm.Query):
     """Extension of default Query class used in SQLAlchemy session queries.
     """
 
-    # Default per_page argument for pagination when per_page not specified.
+    #: Default per_page argument for pagination when per_page not specified.
     DEFAULT_PER_PAGE = 50
 
     @property
     def entities(self):
-        """Return list of entity classes present in query"""
+        """Return list of entity classes present in query."""
         return [e.mapper.class_ for e in self._entities]
 
     @property
     def join_entities(self):
-        """Return list of the joined entity classes present in query"""
+        """Return list of the joined entity classes present in query."""
         return [e.mapper.class_ for e in self._join_entities]
 
     @property
     def all_entities(self):
-        """Return list of entities + join_entities present in query"""
+        """Return list of entities + join_entities present in query."""
         return self.entities + self.join_entities
 
     def _join_eager(self, keys, outerjoin, **kargs):
-        """Helper method for applying join()/outerjoin() with contains_eager()
+        """Helper method for applying ``join()``/``outerjoin()` with
+        ``contains_eager()``.
         """
         alias = kargs.pop('alias', None)
 
@@ -71,11 +79,11 @@ class Query(orm.Query):
         return join(*join_args).options(opt)
 
     def join_eager(self, *keys, **kargs):
-        """Apply `self.join()` + `self.options(contains_eager())`."""
+        """Apply ``join`` + ``self.options(contains_eager())``."""
         return self._join_eager(keys, False, **kargs)
 
     def outerjoin_eager(self, *keys, **kargs):
-        """Apply `self.outerjoin()` + `self.options(contains_eager())`."""
+        """Apply ``outerjoin`` + ``self.options(contains_eager())``."""
         return self._join_eager(keys, True, **kargs)
 
     def _join_load(self, keys, load_type, **kargs):
@@ -88,32 +96,32 @@ class Query(orm.Query):
         return self.options(opt)
 
     def joinedload(self, *keys, **kargs):
-        """Apply joinedload() to `keys`"""
+        """Apply ``joinedload()`` to `keys`."""
         return self._join_load(keys, 'joinedload', **kargs)
 
     def immediateload(self, *keys, **kargs):
-        """Apply immediateload() to `keys`"""
+        """Apply ``immediateload()`` to `keys`."""
         return self._join_load(keys, 'immediateload', **kargs)
 
     def lazyload(self, *keys, **kargs):
-        """Apply lazyload() to `keys`"""
+        """Apply ``lazyload()`` to `keys`."""
         return self._join_load(keys, 'lazyload', **kargs)
 
     def noload(self, *keys, **kargs):
-        """Apply noload() to `keys`"""
+        """Apply ``noload()`` to `keys`."""
         return self._join_load(keys, 'noload', **kargs)
 
     def subqueryload(self, *keys, **kargs):
-        """Apply subqueryload() to `keys`"""
+        """Apply ``subqueryload()`` to `keys`."""
         return self._join_load(keys, 'subqueryload', **kargs)
 
     def load_only(self, *columns):
-        """Apply `load_only()` to query"""
+        """Apply ``load_only()`` to query."""
         obj, columns = get_load_options(*columns)
         return self.options(obj.load_only(*columns))
 
     def defer(self, *columns):
-        """Apply `defer()` to query"""
+        """Apply ``defer()`` to query."""
         obj, columns = get_load_options(*columns)
         opts = obj
         for column in columns:
@@ -121,7 +129,7 @@ class Query(orm.Query):
         return self.options(opts)
 
     def undefer(self, *columns):
-        """Apply `undefer()` to query"""
+        """Apply ``undefer()`` to query."""
         obj, columns = get_load_options(*columns)
         opts = obj
         for column in columns:
@@ -129,16 +137,16 @@ class Query(orm.Query):
         return self.options(opts)
 
     def undefer_group(self, *names):
-        """Apply `undefer_group()` to query"""
+        """Apply ``undefer_group()`` to query."""
         obj, names = get_load_options(*names)
         return self.options(obj.undefer_group(names[0]))
 
     def map(self, func):
-        """Call native `map` on `self.all()`"""
+        """Map `func` to each item returned by :meth:`all`."""
         return [func(item) for item in self.all()]
 
     def reduce(self, func, initial=None, reverse=False):
-        """Reduce `self.all()` using `func`"""
+        """Reduce :meth:`all` using `func`."""
         items = self.all()
 
         if reverse:
@@ -147,27 +155,30 @@ class Query(orm.Query):
         result = initial
         for item in items:
             result = func(result, item)
+
         return result
 
     def reduce_right(self, func, initial=None):
-        """Reduce reversed `self.all()` using `func`"""
+        """Reduce reversed :meth:`all` using `func`."""
         return self.reduce(func, initial=initial, reverse=True)
 
     def pluck(self, column):
-        """Pluck `column` attribute values from `self.all()` results and return
-        as list.
+        """Pluck `column` attribute values from :meth:`all` results and
+        return as list.
         """
         return [getattr(r, column, None) for r in self.all()]
 
     def page(self, page=1, per_page=None):
-        """Return query with limit and offset applied for page"""
+        """Return query with limit and offset applied for page."""
         if per_page is None:
             per_page = self.DEFAULT_PER_PAGE
 
         return self.limit(per_page).offset((page - 1) * per_page)
 
     def paginate(self, page=1, per_page=None, error_out=True):
-        """Return Pagination instance using already defined query parameters"""
+        """Return :class:`Pagination` instance using already defined query
+        parameters.
+        """
         if error_out and page < 1:
             raise IndexError
 
@@ -190,30 +201,50 @@ class Query(orm.Query):
 
 
 class QueryModel(Query):
-    """Class used for default query property class. It's assumed that is how
-    this class will be used so there may be cases where some of its functions
-    won't work properly outside that context.
+    """Class used for default query property class for ``mymanager.query``,
+    ``mymanager.session.query``, and ``MyModel.query``. Can be used in other
+    libraries/implementations when creating a session::
+
+        from sqlalchemy import orm
+
+        from alchy import QueryModel
+        # or if not using as query property
+        # from alchy import Query
+
+
+        session = orm.scoped_session(orm.sessionmaker())
+        session.configure(query_cls=QueryModel)
+
+    **NOTE:** If you don't plan to use the query class as a query property,
+    then you can use the :class:`Query` class instead since it won't include
+    features that only work within a query property context.
+
+    Attributes:
+
+        __search_filters__: All available search filter functions indexed by a
+            canonical name which will be referenced in advanced/simple search.
+            All filter functions should take a single value and return an
+            SQLAlchemy filter expression, i.e.,
+            ``{key: lambda value: Model.column_name.contains(value)}``
+
+        __advanced_search__: Advanced search models search by named parameters.
+            Generally found on advanced search forms where each field maps to a
+            specific database field that will be queried against. If defined as
+            a list, each item should be a key from :attr:`__search_filters__`.
+            The matching :attr:`__search_filters__` function will be used in
+            the query. If defined as a dict, it should have the same format as
+            :attr:`__search_filters__`.
+
+        __simple_search__: Simple search models search by phrase (like Google
+            search). Defined like :attr:`__advanced_search__`.
+
+        __order_by__: Default order-by to use when
+            :attr:`alchy.model.ModelBase.query` used.
     """
 
-    # All available search filter functions indexed by a canonical name which
-    # will be referenced in advanced/simple search. All filter functions should
-    # take a single value and return an SQLAlchemy filter expression.
-    # I.e. {key: lambda value: Model.column_name.contains(value)}
     __search_filters__ = {}
-
-    # Advanced search models search by named parameters. Generally found on
-    # advanced search forms where each field maps to a specific database field
-    # that will be queried against. If defined as a list, each item should be
-    # a key from __search_filters__. The matching __search_filters__ function
-    # will be used in the query. If defined as a dict, it should have the same
-    # format as __search_filters__.
     __advanced_search__ = []
-
-    # Simple search models search by phrase (like Google search). Defined like
-    # __advanced_search__.
     __simple_search__ = []
-
-    # Default order by to use.
     __order_by__ = None
 
     @property
@@ -222,14 +253,15 @@ class QueryModel(Query):
         return self.entities[0]
 
     def get_search_filters(self, keys):
-        """Return __search_filters__ filtered by keys."""
+        """Return :attr:`__search_filters__` filtered by keys."""
         if isinstance(keys, dict):
             return keys
         else:
             return dict([(key, self.__search_filters__[key]) for key in keys])
 
     def advanced_filter(self, search_dict=None):
-        """Return the compiled advanced search filter mapped to search_dict."""
+        """Return the compiled advanced search filter mapped to `search_dict`.
+        """
         if search_dict is None:  # pragma: no cover
             search_dict = {}
 
@@ -242,7 +274,8 @@ class QueryModel(Query):
         return and_(*term_filters)
 
     def simple_filter(self, search_terms=None):
-        """Return the compiled simple search filter mapped to search_terms."""
+        """Return the compiled simple search filter mapped to `search_terms`.
+        """
         if search_terms is None:  # pragma: no cover
             search_terms = []
 
@@ -278,7 +311,7 @@ class QueryModel(Query):
         # users, we may actually have more than that many records since we're
         # joining on many records from the user keywords table.
         original = (self.lazyload('*')
-                    .load_only(*self.Model.primary_attrs)
+                    .load_only(*self.Model.primary_attrs())
                     .distinct())
 
         # Use the original query so that we preserve joins and where
@@ -325,14 +358,15 @@ class QueryModel(Query):
             subquery = model_query.subquery()
             query = query.join(
                 subquery, join_subquery_on_columns(subquery,
-                                                   self.Model.primary_keys))
+                                                   self.Model.primary_keys()))
 
         return query
 
 
 class QueryProperty(object):
     """Query property accessor which gives a model access to query capabilities
-    via Model.query which is equivalent to session.query(Model)
+    via :attr:`alchy.model.ModelBase.query` which is equivalent to
+    ``session.query(Model)``.
     """
     def __init__(self, session):
         self.session = session
@@ -355,57 +389,54 @@ class QueryProperty(object):
 
 
 class Pagination(object):
-    """
-    Internal helper class returned by `BaseQuery.paginate`.  You
-    can also construct it from any other SQLAlchemy query object if you are
-    working with other libraries.  Additionally it is possible to pass `None`
-    as query object in which case the `prev` and `next` will
-    no longer work.
+    """Internal helper class returned by :meth:`Query.paginate`. You can also
+    construct it from any other SQLAlchemy query object if you are working with
+    other libraries. Additionally it is possible to pass ``None`` as query
+    object in which case the `prev` and `next` will no longer work.
     """
 
     def __init__(self, query, page, per_page, total, items):
-        # the unlimited query object that was used to create this pagination
-        # object
+        #: The query object that was used to create this pagination object.
         self.query = query
 
-        # the current page number (1 indexed)
+        #: The current page number (1 indexed).
         self.page = page
 
-        # the number of items to be displayed on a page.
+        #: The number of items to be displayed on a page.
         self.per_page = per_page
 
-        # the total number of items matching the query
+        #: The total number of items matching the query.
         self.total = total
 
-        # the items for the current page
+        #: The items for the current page.
         self.items = items
 
-        # the total number of pages
         if self.per_page == 0:
             self.pages = 0
         else:
+            #: The total number of pages.
             self.pages = int(ceil(self.total / float(self.per_page)))
 
-        # number of the previous page
+        #: Number of the previous page.
         self.prev_num = self.page - 1
 
-        # True if a previous page exists
+        #: True if a previous page exists.
         self.has_prev = self.page > 1
 
-        # number of the next page
+        #: Number of the next page.
         self.next_num = self.page + 1
 
-        # True if a next page exists
+        #: True if a next page exists.
         self.has_next = self.page < self.pages
 
     def prev(self, error_out=False):
-        """Returns a `Pagination` object for the previous page."""
+        """Returns a :class:`Pagination` object for the previous page."""
         assert self.query is not None, \
             'a query object is required for this method to work'
         return self.query.paginate(self.page - 1, self.per_page, error_out)
 
     def next(self, error_out=False):
-        """Returns a `Pagination` object for the next page."""
+        """Returns a :class:`Pagination` object for the next page."""
         assert self.query is not None, \
             'a query object is required for this method to work'
         return self.query.paginate(self.page + 1, self.per_page, error_out)
@@ -428,3 +459,27 @@ def get_load_options(*columns):
         obj = orm
 
     return (obj, columns)
+
+
+def base_columns_from_subquery(subquery):
+    """Return non-aliased, base columns from subquery."""
+    # base_columns is a set so we need to cast to list.
+    return [(column, list(column.base_columns))
+            for column in subquery.c.values()]
+
+
+def join_subquery_on_columns(subquery, columns):
+    """Return join-on condition which maps subquery's columns to columns."""
+    subquery_base_columns = base_columns_from_subquery(subquery)
+
+    join_on = []
+    for subquery_column, base_columns in subquery_base_columns:
+        # Don't support joining to subquery column with more than 1 base
+        # column.
+        if len(base_columns) == 1 and base_columns[0] in columns:
+            join_on.append(subquery_column == base_columns[0])
+
+    if join_on:
+        return and_(*join_on)
+    else:  # pragma: no cover
+        return None
